#include "libcDAG.hpp"

cDAG_Graph cDAG_Create_graph() {

#ifdef HAVE_PGT
	PGT_Rec(0); // to start a new epoch, which times the graph-building
#endif

	cDAG_Graph newg = (cDAG_Graph) malloc(sizeof(struct graph));

	newg->alloc_vertexes = 10;
	newg->vertexes = (struct vertex*) malloc(newg->alloc_vertexes * sizeof(struct vertex));
	newg->num_vertexes = 0;
	newg->do_coll_detection = 1;
	newg->scratchpad_size = 0;
	newg->alloc_ops = 10;
	newg->ops = (struct op*) malloc(newg->alloc_ops * sizeof(struct op));
	newg->num_ops = 0;

	return newg;

}

/**
 * This is the same as cDAG_Create_graph(), it is for calls from inside the collective optimization
 */

cDAG_Graph _cDAG_Create_graph_internal() {

	cDAG_Graph newg = (cDAG_Graph) malloc(sizeof(struct graph));

	newg->alloc_vertexes = 10;
	newg->vertexes = (struct vertex*) malloc(newg->alloc_vertexes * sizeof(struct vertex));
	newg->num_vertexes = 0;
	newg->do_coll_detection = 1;
	newg->scratchpad_size = 0;
	newg->alloc_ops = 10;
	newg->ops = (struct op*) malloc(newg->alloc_ops * sizeof(struct op));
	newg->num_ops = 0;

	return newg;

}


int cDAG_FreeGraph(cDAG_Graph g) {

  for (int i=0; i<g->num_vertexes; i++) {
	if (g->vertexes[i].type == T_OP) free(((NBC_Args_op*) (g->vertexes[i].args))->args);
	free(g->vertexes[i].adj);
	free(g->vertexes[i].args);
  }
  free(g->ops);
  free(g->vertexes);
  free(g);

  return 0;

}

/**
 * adds a vertex to a cDAG_Graph.
 * @param [g]: The cDAG_Graph to add a new vertex to
 * @param [type]: The type of the vertex that should be added (T_SEND, T_RECV or T_OP)
 * @param [args]: The operation specific arguments for the newly added vertex
 */

int add_vertex(cDAG_Graph g, char type, void* args) {
 
  g->num_vertexes++;

  if(g->num_vertexes > g->alloc_vertexes) {
    g->alloc_vertexes *= 2;
    g->vertexes = (struct vertex*) realloc(g->vertexes, g->alloc_vertexes*sizeof(struct vertex));
  }

  int ind=g->num_vertexes-1;
  g->vertexes[ind].adj=(int*)malloc(2*sizeof(int));
  g->vertexes[ind].num_adj=0;
  g->vertexes[ind].alloc_adj=2;
  g->vertexes[ind].num_rev=0;

  g->vertexes[ind].type=type;
  g->vertexes[ind].args = malloc(type_get_size(type));
  memcpy(g->vertexes[ind].args, args, type_get_size(type));

  return ind;

}

void cDAG_Requires(cDAG_Graph g, cDAG_Vertex prereq, cDAG_Vertex target) {

  int ind = prereq;

  g->vertexes[ind].num_adj++;
  if(g->vertexes[ind].num_adj > g->vertexes[ind].alloc_adj) {
    g->vertexes[ind].alloc_adj *= 2;
    g->vertexes[ind].adj = (int*)realloc(g->vertexes[ind].adj, g->vertexes[ind].alloc_adj*sizeof(int));
  }

  int adjind = g->vertexes[ind].num_adj-1;
  g->vertexes[ind].adj[adjind]=target;

  g->vertexes[target].num_rev++;

  return;

}

